@@ -37,11 +37,15 @@ import yaml
 
 
 
-__version__ = '0.2'
+__version__ = '0.3'
 
 
 LOCALES_DIR = 'locales'
 
+class InvalidLocale(Exception):
+    pass
+
+
 # For parsing raw 'User-Agent' headers
 _language_re = re.compile(
     r'(?:;\s*|\s+)(\b\w{2}\b(?:-\b\w{2}\b)?)\s*;|'
@@ -119,15 +123,9 @@ class I18n(object):
         `Babel.Locale` and the default timezone as a `pytz.timezone` object.
 
         """
-        default_locale = default_locale or 'en'
-        if isinstance(default_locale, tuple):
-            self.default_locale = Locale(*default_locale)
-        else:
-            default_locale = default_locale.replace('_', '-')
-            self.default_locale = Locale.parse(default_locale, sep='-')
-
-        default_timezone = default_timezone or 'utc'
-        self.default_timezone = timezone(default_timezone)
+        default_locale = normalize_locale(default_locale) or Locale('en')
+        self.default_locale = default_locale
+        self.default_timezone = timezone(default_timezone or 'utc')
 
 
     def list_available_languages(self):
@@ -332,7 +330,7 @@ class I18n(object):
 
         """
         key = str(key)
-        locale = locale or self.get_locale()
+        locale = normalize_locale(locale) or self.get_locale()
         value = self.key_lookup(key, locale)
         if not value:
             return self.markup('<missing:%s>' % (key, ))
@@ -443,7 +441,7 @@ class I18n(object):
         """Internal helper that formats the date.
 
         """
-        locale = locale or self.get_locale()
+        locale = normalize_locale(locale) or self.get_locale()
         extra = {}
         if formatter is not dates.format_date and rebase:
             extra['tzinfo'] = tzinfo or self.get_timezone()
@@ -561,7 +559,7 @@ class I18n(object):
         named `timedeltaformat`.
 
         """
-        locale = locale or self.get_locale()
+        locale = normalize_locale(locale) or self.get_locale()
         if isinstance(datetime_or_timedelta, d.datetime):
             datetime_or_timedelta = d.datetime.utcnow() - datetime_or_timedelta
         return dates.format_timedelta(datetime_or_timedelta, granularity,
@@ -582,7 +580,7 @@ class I18n(object):
         named `numberformat`.
         
         """
-        locale = locale or self.get_locale()
+        locale = normalize_locale(locale) or self.get_locale()
         return numbers.format_number(number, locale=locale)
 
 
@@ -602,7 +600,7 @@ class I18n(object):
         named `decimalformat`.
         
         """
-        locale = locale or self.get_locale()
+        locale = normalize_locale(locale) or self.get_locale()
         return numbers.format_decimal(number, format=format, locale=locale)
 
 
@@ -624,7 +622,7 @@ class I18n(object):
         named `currencyformat`.
         
         """
-        locale = locale or self.get_locale()
+        locale = normalize_locale(locale) or self.get_locale()
         return numbers.format_currency(
             number, currency, format=format, locale=locale
         )
@@ -646,7 +644,7 @@ class I18n(object):
         named `percentformat`.
 
         """
-        locale = locale or self.get_locale()
+        locale = normalize_locale(locale) or self.get_locale()
         return numbers.format_percent(number, format=format, locale=locale)
 
 
@@ -666,10 +664,20 @@ class I18n(object):
         named `scientificformat`.
 
         """
-        locale = locale or self.get_locale()
+        locale = normalize_locale(locale) or self.get_locale()
         return numbers.format_scientific(number, format=format, locale=locale)
 
 
+def normalize_locale(locale):
+    if isinstance(locale, Locale):
+        return locale
+    if isinstance(locale, basestring):
+        locale = locale.replace('_', '-')
+        return Locale.parse(locale, sep='-')
+    if isinstance(locale, tuple):
+        return Locale(*locale)
+    return
+        
 
 def get_werkzeug_preferred_languages(request):
     """Return a list of preferred languages from a `werkzeug.wrappers.Request`
